unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, XPMan, ExtCtrls, Grids, ValEdit, StrUtils,
  {Engine's units}
  Sys, Objects, Images, Dagger, Strings, Win32,
  TgaModule, PngModule, BmpModule, JpgModule
  ;

type
  TForm1 = class(TForm)
    XPMan: TXPManifest;
    GroupBox1: TGroupBox;
    Paint: TPaintBox;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    List: TValueListEditor;
    Button1: TButton;
    Button2: TButton;
    CBFilter: TCheckBox;
    LabActual: TLabel;
    LabRes: TLabel;
    Button3: TButton;
    Button4: TButton;
    CBRatio: TCheckBox;
    OpenD: TOpenDialog;
    SaveD: TSaveDialog;
    Button5: TButton;
    Button6: TButton;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PaintPaint(Sender: TObject);
    procedure CBRatioClick(Sender: TObject);
    procedure CBFilterClick(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }

    Drawing: TImageData;
    Image: TImageData;
    AFrame: Long;
    AName: string;
    AMod: TImageModule;
    Zoom: Long;

    CifModule: TCifModule;
    ImgModule: TImgModule;
    TextureModule: TTextureModule;

    procedure LoadImage(const FName: string);
    procedure UpdateAll;
    procedure SetFrame(Idx: Long);

  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.Button1Click(Sender: TObject);
var
  Ext: string;
  IMod: TImageModule;
begin
  if OpenD.Execute then
  begin
    Ext := ExtractFileExt(OpenD.FileName);
    if SameStr(Ext, '.img') then
      AMod := ImgModule
    else
      if SameStr(Ext, '.cif') then
        AMod := CifModule
      else
        if AnsiContainsText(OpenD.FileName, 'texture') then
          AMod := TextureModule;
    LoadImage(OpenD.FileName);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  CifModule := TCifModule.Create;
  ImgModule := TImgModule.Create;
  TextureModule := TTextureModule.Create;
  Zoom := 1;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  CifModule.Free;
  ImgModule.Free;
  TextureModule.Free;
end;

procedure TForm1.LoadImage(const FName: string);
var
  I: Long;
begin
  if AMod = nil then
    Exit;

  AName := FName;
  try
    AMod.DecodeFile(FName, Image);
    CloneImage(Image, Drawing);
  except
    RaiseError(ErrString(Self, 'LoadImage', 'Error when loading image "'
      + FName + '"!'));
    AName := '';
  end;
  CBFilterClick(Self);
  UpdateAll;
end;

procedure TForm1.SetFrame(Idx: Long);
begin
  AFrame := Iff(Idx < 0, Image.LevelCount - 1, Idx);
  AFrame := Iff(AFrame > Image.LevelCount - 1, 0, AFrame);
  LabActual.Caption := Format('Actual Frame: %d/%d', [AFrame + 1,
    Image.LevelCount]);
  LabRes.Caption := Format('Frame Resolution: %dx%d',
    [Image.Levels[AFrame].Width, Image.Levels[AFrame].Height]);

  Paint.OnPaint(Self);
end;

procedure TForm1.UpdateAll;

  function FileGetSize(const FileName: string): Integer;
  var
    SearchRec: TSearchRec;
  begin
    Result := -1;
    try
      if FindFirst(FileName, faAnyFile, SearchRec) = 0 then
      begin
        Result := SearchRec.Size;
        FindClose(SearchRec);
      end;
    except
      Result := 0;
    end;
  end;

begin
  List.Values['Name'] := ExtractFileName(AName);
  List.Values['Size'] := IntToStr(FileGetSize(AName)) + ' Bytes';
  List.Values['Frames'] := IntToStr(Image.LevelCount);
  List.Values['Format'] := 'ifIndex8';
  List.Values['Type'] := Copy(AMod.ClassName, 2,
    Length(AMod.ClassName) - 1);

  Paint.Canvas.FillRect(Paint.Canvas.ClipRect);

  SetFrame(0);
end;

procedure TForm1.PaintPaint(Sender: TObject);
var
  BMP: TBitmap;
begin
  if AFrame > Image.LevelCount - 1 then
    Exit;

  Paint.Canvas.FillRect(Paint.Canvas.ClipRect);
  BMP := Win32.ImageToBitmap(Drawing, AFrame);
  if CBRatio.Checked then
    Paint.Canvas.StretchDraw(Paint.ClientRect, BMP)
  else
  begin
    Paint.Canvas.StretchDraw(Rect(0, 0,
      BMP.Width * Zoom, BMP.Height * Zoom), BMP);
  end;
  BMP.Free;
end;

procedure TForm1.CBRatioClick(Sender: TObject);
begin
  Paint.OnPaint(nil);
end;

procedure TForm1.CBFilterClick(Sender: TObject);
begin
  if CBFilter.Checked then
    FilterBilinear(Drawing)
  else
    CloneImage(Image, Drawing);
  Paint.OnPaint(nil);
end;

procedure TForm1.Button5Click(Sender: TObject);
begin
  Zoom := Iff(Zoom < 10, Zoom + 1, 10);
  Paint.OnPaint(nil);
end;

procedure TForm1.Button6Click(Sender: TObject);
begin
  Zoom := Iff(Zoom > 1, Zoom - 1, 1);
  Paint.OnPaint(nil);
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
  SetFrame(AFrame + 1);
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  SetFrame(AFrame - 1);
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  Temp: TImageData;
  I: Long;
  SMod: TImageModule;
  Ext: string;
begin
  if not SaveD.Execute then
    Exit;

  SMod := nil;
  Ext := ExtractFileExt(SaveD.FileName);
  SaveD.FileName := ChangeFileExt(SaveD.FileName, '');

  case SaveD.FilterIndex of
    1: SMod := TPngModule.Create;
    2: SMod := TJpgModule.Create;
    3: SMod := TTgaModule.Create;
    4: SMod := TBmpModule.Create;
  end;

  for I := 0 to Image.LevelCount - 1 do
  begin
    Temp := Image;
    Temp.LevelCount := 1;
    Temp.Levels[0] := Image.Levels[I];
    SMod.EncodeFile(Temp, SaveD.FileName + IntToStr(I) + Ext);
  end;
  SMod.Free;
end;

end.

